#!/usr/bin/env python
#^#
# OVSDB.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
"""
~-- data structure design --~

!-- virtual machines --!
vm.db := {vm_path: { zip(vm_attr_list, []) }} 
vm_attr_list := [ 
	vtech, host_srv, 
	cpu_num, cpu_speed, mem_total, 
        packaged, status, readlock,
	nics,
        agt_addr, agt_username, agt_passwd, agt_port,
]
vm["nics"] := {dev_name : {zip([inet_type, ip, netmask, gateway], [])}}

#N.B.
    *) inet_type in (DHCP, STATIC, LOOP)
    *) status in (DOWN, RUNNING, ERROR, SUSPEND, READLOCK, PROCESSING)
       PROCESSING is kind of WRITELOCK

!-- vm_name to vm_path mapping --!
loc.db := {vm_name: vm_path}

!-- mac ip mapping --!
macip.db := {mac : ip}

#N.B.
    *) mac -> ip, register when vm startup

!-- physical servers --!
srv.db := {hostname: {zip(srv_attr_list, [])} }
srv_attr_list = [
	is_shared, srv_role,
	cpu_num, cpu_speed, mem_total, mem_free, cpu_idle, 
	agt_addr, agt_username, agt_passwd, agt_port, agt_url
]

#N.B.
    *) cpu_avail = cpu_num * cpu_speed * cpu_idle
    *) srv_role, in (xen, utility, site) can be combination of these value
    *) hostname, could be an ip, but should maintain consistently, i.e. 
       the same server registered with ip & hostname will be considered
       as 2 different entries 
    *) status in (RUNNING, DOWN, UNKNOWN)

!--  --!
#N.B.  
    *) cpu in MHz, mem in MB
"""

from OVSConstants import DEFAULT_AGENT_USERNAME, DEFAULT_AGENT_PASSWD, DEFAULT_AGENT_PORT
from OVSCDS import CDS
from OVSXUtility import find_real_path
from os.path import basename, exists 

_g_vm_default = {
    "vtech": "xen", 
    "status": "DOWN", 
    "ip":"", 
    "agt_addr":"", 
    "agt_username": DEFAULT_AGENT_USERNAME,
    "agt_passwd": DEFAULT_AGENT_PASSWD,
    "agt_port": DEFAULT_AGENT_PORT,
    "readlock": 0,
    "host_srv": "",
    "real_path": "",
    }

#----------------------------<> vm <>----------
def db_get_vm(vm_path):
    cds = CDS("vm", "vmp")
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
        else:
            cds.db_vm[vm_path] = _g_vm_default.copy()
            vm = cds.db_vm[vm_path]
            #name->path index
            vm_name = basename(vm_path)
            cds.db_vmp[vm_name] = vm_path
    finally:
        cds.close()
    return vm

def db_update_vm(vm_path, d):
    cds = CDS("vm", "vmp")
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
            vm.update(d)
        else: 
            vm = _g_vm_default
            vm.update(d)
        cds.db_vm[vm_path] = vm
        #name->path index
        vm_name = basename(vm_path)
        cds.db_vmp[vm_name] = vm_path
    finally:
        cds.close()
    return vm

def db_del_vm(vm_path):
    cds = CDS("vm", "vmp")
    try:
        if vm_path in cds.db_vm:
            del cds.db_vm[vm_path]
        #name->path index
        vm_name = basename(vm_path)
        if vm_name in cds.db_vmp:
            del cds.db_vmp[vm_name]
    finally:
        cds.close()

def db_chkset_vm_status(vm_path, status_before, status_after):
    cds = CDS("vm", "vmp")
    try:
        if vm_path in cds.db_vm:
            vm = cds.db_vm[vm_path]
        else:
            cds.db_vm[vm_path] = _g_vm_default.copy()
            vm = cds.db_vm[vm_path]
            #name->path index
            vm_name = basename(vm_path)
            cds.db_vmp[vm_name] = vm_path

        if status_before:
            if vm["status"] not in status_before:
                errmsg = "vm('%s') status ('%s') not in %s" % \
                        (vm_path, vm['status'], repr(status_before))
                raise Exception(errmsg)
            #accquire read lock
            elif status_after == "READLOCK":
                #vm status may out of sync with lock, resync
                if vm["status"] == "DOWN": 
                    vm["readlock"] = 0
                vm["readlock"] += 1
                vm['status'] = status_after
            #release read lock
            #include DOWN in status_before, just be more torlerance 
            elif vm["status"] in ("READLOCK", "DOWN") and status_after == "DOWN":
                if vm["readlock"] > 0: vm["readlock"] -= 1
                if vm["readlock"] == 0: vm['status'] = "DOWN"
            else: 
                vm['status'] = status_after
        else: 
            #if not status_before
            vm['status'] = status_after
        
        cds.db_vm[vm_path] = vm
    finally:
        cds.close()
    
#----------------------------<> server <>----------
def db_get_srv(hostname):
    cds = CDS("srv")
    try:
        if hostname in cds.db_srv: 
            srv = cds.db_srv[hostname]
        else:
            raise Exception("srv('%s') not found in db!" % hostname)
    finally: 
            cds.close()

    return srv

def db_update_srv(hostname, d):
    cds = CDS("srv")
    try:
        if hostname in cds.db_srv:
            srv = cds.db_srv[hostname]
            srv.update(d)
        else:
            srv = d
        cds.db_srv[hostname] = srv
    finally:
        cds.close()

def db_del_srv(hostname):
    cds = CDS("srv")
    try:
        if hostname in cds.db_srv:
            del cds.db_srv[hostname]
    finally:
        cds.close()

#------------------------------<> vm mac/ip <>----------
def db_get_mac_ip(mac):
    mac = mac.upper()
    cds = CDS("macip")
    try:
        if mac in cds.db_macip: 
            ip = cds.db_macip[mac]
        else: 
            ip = ""
    finally:
        cds.close()
    return ip

def db_remove_mac_ip(mac):
    mac = mac.upper()
    cds = CDS("macip")
    try:
        if mac in cds.db_macip:
            del cds.db_macip[mac]
    finally:
        cds.close()

#------------------------------<> vm path mapping <>------
def db_get_real_vm_path(vm_path):
    real_path = ""
    vm = db_get_vm(vm_path)
    if "real_path" in vm and vm["real_path"] and exists(vm["real_path"]):
        real_path = vm["real_path"]   
    else:
        #if not exist, find_real_path will raise Exception
        real_path = find_real_path(vm_path)
        db_update_vm(vm_path, {"real_path": real_path})
    return real_path

def db_get_vm_path(vm_name):
    vm_path = ""
    cds = CDS("vm", "vmp")
    try:
        if vm_name in cds.db_vmp:
            vm_path = cds.db_vmp[vm_name]
        else: #init db_vmp with db_vm
            for vm_path in cds.db_vm:
                name = basename(vm_path)
                if name not in cds.db_vmp:
                    cds.db_vmp[name] = vm_path
            if vm_name in cds.db_vmp:
                vm_path = cds.db_vmp[vm_name]
    finally:
        cds.close()
    return vm_path
