#!/usr/bin/python
#^#
# OVSServerSelection.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
from OVSServerProxy import ServerProxy
from OVSCommons import exposed, fmt_err_msg, parse_result
from OVSCDS import CDS
from OVSDB import db_get_srv, db_update_srv, db_get_vm, db_get_real_vm_path

#----------------------------------------<> helper functions <>----
def select_tgt(l, field, goal="max"):
    """
    l is a list of (k, d) and d is a dict with (field in d)
    """
    if not l: return
    (vk, vd) = l[0]
    if goal == "max":
        for (k,d) in l[1:]:
            if vd[field] < d[field]:
                (vk, vd) = (k, d)
    else:
        for (k,d) in l[1:]:
            if vd[field] > d[field]:
                (vk, vd) = (k, d)
    return vk

#----------------------------------------<> <>----
def collect_srv_stat(candidates=None):
    """
    collect server statistics data throu agent
    save collected info to srv.db
    
    @para candidates	list of hostnames
    """
    logger = logging.getLogger("ovs.performance")

    if not candidates:
	cds = CDS("srv")
        try: 
            candidates = cds.db_srv.keys()
	finally:
            cds.close()
    
    for hostname in candidates:
	try:
	    srv = db_get_srv(hostname)
	    sp = ServerProxy(srv['agt_url'])
            #TODO: combine these 2 rpc calls into 1
	    rs = sp.sys_vmstat_info()
	    rs = parse_result(rs)
	    srv["cpu_idle"] = float(rs["cpu_idle"])
            srv_vtech = "xen" #default to xen
            vtechs = ("xen",)
            for vtech in vtechs:
                if vtech in srv["srv_role"]:
                    srv_vtech = vtech
                    break
            rs = getattr(sp, "%s_mem_free" % vtech)()
            rs = parse_result(rs)
            srv["mem_free"] = int(rs["mem_free"])
	    #FIXME: any better algorithm to describe the cpu power?
	    if "cpu_speed" not in srv:
		rs = sp.sys_cpu_info()
		if not rs.starswith("success"): raise Exception(rs)
		rs = parse_result(rs)
		srv["cpu_num"] = rs["cpu_num"]
		srv["cpu_speed"] = rs["bogomips"]
	    srv["cpu_avail"] = srv["cpu_num"] * srv["cpu_speed"] * srv["cpu_idle"]
	    srv["status"] = "RUNNING"
	except KeyError, e:
	    #when unregister one of the candidates occurs before this loop
	    errmsg = fmt_err_msg(e)
	    logger.warning("collect_srv_stat: srv('%s') removed?=>%s" % \
		    (hostname, errmsg))
            continue
	except Exception, e:
	    errmsg = fmt_err_msg(e)
	    logger.warning("collect_srv_stat: srv('%s') error=>%s" % \
		    (hostname, errmsg))
	    srv["status"] = "UNKNOWN"

	db_update_srv(hostname, srv)

    logger.info("collect_srv_stat: success. %s" % repr(candidates))

def select_tgt_host(vm_path, any=False, excludes=[], with_attr={}):
    """
    select server for running the vm
    ~-- balancing strategy --~
    1) enough memory to accommodate the new vm
    2) the fast one in candidates 1)
    this is only one of the strategies, other possiblities for e.g.:
    *) chose the one with memory best-fit size of vm memory
    *) ...

    @param excludes 
    @param with_attr
    these 2 param are for migrate_vm, 
    1) excludes itself or some specific server
    2) server arch must be the same
    
    """
    logger = logging.getLogger("ovs.operation")
    #---------------------------------------------------------------------
    def is_subset(d1, d2):
	try:
	    for k in d1:
		if d1[k] != d2[k]: return False
	except:
	    return False
	return True
    #---------------------------------------------------------------------
    tgt_host = None
    cds = CDS("vm", "srv")
    try:
	vm = cds.db_vm[vm_path]
	vtech = vm["vtech"]
	candidates = [ hostname for (hostname, srv) in cds.db_srv.items() 
			if vtech in srv["srv_role"] and 
			    srv["status"] == "RUNNING" and 
			    is_subset(with_attr, srv) ]
	if excludes: candidates = list(set(candidates) - set(excludes))
    finally: cds.close()

    if not candidates: 
	raise Exception("no server of type '%s'." % vm['vtech'])

    if any: 
	#any server not depends on vm memory size
	#for e.g. vm installation
	srvs = {}
	collect_srv_stat(candidates)
	cds = CDS("srv")
	try:
	    for hostname in candidates:
		srv = cds.db_srv[hostname]
		if srv["status"] == "RUNNING":
		    srvs[hostname] = srv
	finally: cds.close()
	#here we choose server with max memory
	tgt_host = select_tgt(srvs.items(), "mem_free")
	sp = ServerProxy(srvs[tgt_host]['agt_url'])

	logger.info("select_tgt_host: vm('%s') will run on srv('%s')" % \
		(vm_path, tgt_host))
	return (tgt_host, sp)
    #FIXME:
    #infact, if the vm claim to use 1G ram, the xen server 
    #won't allocate so much ram, it will dynamically allocate memory
    #so a host server with 2 G mem may accommodate 3 vms each with 2 G ram 
    (hostname, sp) = select_tgt_host(vm_path, any=True)
    real_vm_path = db_get_real_vm_path(vm_path)
    rs = getattr(sp, "%s_get_memory" % vm["vtech"])(real_vm_path)
    if rs.startswith("success"):
	mem_size = int(parse_result(rs)["mem_size"])
	logger.debug("select_tgt_host: vm('%s')=>mem_size=%d" % (vm_path, mem_size))
    else:
	raise Exception(rs)
    
    srvs = {}
    collect_srv_stat(candidates)
    cds = CDS("srv")
    try:
	for hostname in candidates:
	    srv = cds.db_srv[hostname]
	    if (srv["status"] == "RUNNING") and (srv["mem_free"] > mem_size):
		srvs[hostname] = srv 
    finally: cds.close()

    if not srvs: 
	raise Exception("no server selected to run vm('%s') mem_size=%d" % \
	    (vm_path, mem_size))
    #FIXME:we change our police to run vm on srv with max free mem.
    tgt_host = select_tgt(srvs.items(), "mem_free")
    sp = ServerProxy(srvs[tgt_host]['agt_url'])

    logger.info("select_tgt_host: vm('%s') will run on srv('%s')" % \
	    (vm_path, tgt_host))
    return (tgt_host, sp)

def get_utility_server():
    """
    there may be multiple utility servers.
    balancing policy is 'the one with the maximum cpu resources'.
    """
    #FIXME: should we add another db to store utility,xen,site as index?
    cds = CDS("srv")
    try:
        candidates = [k for k in cds.db_srv if "utility" in cds.db_srv[k]["srv_role"]]
    finally:
        cds.close()

    if not candidates: 
	raise Exception("no available utility server!")

    if len(candidates) == 1: 
	tgt_host = candidates[0]
	srv = db_get_srv(tgt_host)
    else:
	collect_srv_stat(candidates)
	cds = CDS("srv")
        try:
            l = [ (hostname, srv) for (hostname, srv) in cds.db_srv.items() 
                    if (hostname in candidates) and (srv["status"] == "RUNNING") ]
	finally: 
            cds.close()
        if not l: raise Exception("no available utility server!")
        tgt_host = select_tgt(l, "cpu_avail")
        srv = dict(l)[tgt_host]

    sp = ServerProxy(srv['agt_url'])
    return (tgt_host, sp)

def select_one_server(vtech):
    """
    get one server of type vtech, to get cfg etc lite operation
    """
    vtech = vtech.lower().strip()
    tgt_host = None
    cds = CDS("srv")
    candidates = [ hostname for (hostname, srv) in cds.db_srv.items() 
		    if (vtech in srv["srv_role"]) and (srv["status"] == "RUNNING") ]
    cds.close()
    if not candidates:
	raise Exception("no server of type '%s'." % vtech)
    
    tgt_host = candidates[0]
    srv = db_get_srv(tgt_host)
    sp = ServerProxy(srv['agt_url'])
    return (tgt_host, sp)

def get_vm_host(vm_path):
    """
    get vm host server from site db
    """
    vm = db_get_vm(vm_path)
    host_srv = vm['host_srv']
    if not host_srv.strip(): 
        raise Exception("cannot find host server for vm('%s')" % vm_path)
    try:
	srv = db_get_srv(host_srv)
    except Exception, e:
	raise Exception("get host server('%s') for vm('%s') failed! => %s" % \
                (host_srv, vm_path, fmt_err_msg(e)))
    sp = ServerProxy(srv['agt_url'])

    return (host_srv, sp)

