#!/usr/bin/env python
#^#
# OVSSiteDB.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import logging
from os.path import join, basename
from OVSXUtility import find_real_path
from OVSServerSelection import collect_srv_stat, get_utility_server
from OVSServerProxy import ServerProxy
from OVSDB import db_get_vm, db_update_vm, db_del_vm, db_get_real_vm_path
from OVSDB import db_get_srv, db_update_srv, db_del_srv
from OVSDB import db_get_mac_ip
from OVSConstants import DEFAULT_AGENT_USERNAME, DEFAULT_AGENT_PASSWD, DEFAULT_AGENT_PORT, OVS_BASE
from OVSCommons import exposed, fmt_err_msg, parse_result, chk_vm_path, host_alive
from OVSCDS import CDS

#------------------<> register/unregister server/vm <>----
@exposed
def register_server(hostname, srv_role, is_shared=True, 
	agt_username=DEFAULT_AGENT_USERNAME, 
	agt_passwd=DEFAULT_AGENT_PASSWD, 
	agt_port=DEFAULT_AGENT_PORT, 
        ssl_enabled=False):
    """
    NOTE: 
      when register a server, agent on the server must be up and running
    for collecting server infomation
    
    @para srv_role  "site", "xen" or "utility"
    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	srv_role = srv_role.lower()
	if "site" in srv_role:
	    cds = CDS("srv")
	    try:
		candidates = [host for (host, srv) in cds.db_srv.items() 
			if "site" in srv["srv_role"] ]
	    finally: cds.close()

	    if len(candidates) > 1:
		raise Exception("More than one server pool master exists: %s" % repr(candidates))
	    if len(candidates) == 1 and candidates[0] != hostname:
		raise Exception("Master pool server already exists: %s" % candidates[0])

	srv = {
	    #is in shared server pool, vm can be balanced to this server.
	    "is_shared": is_shared, 
	    "srv_role": srv_role,
	    "agt_addr": hostname,
	    "agt_username": agt_username,
	    "agt_passwd": agt_passwd,
	    "agt_port": agt_port,
            "agt_url": "http%s://%s:%s@%s:%s" % \
                (ssl_enabled and "s" or "", agt_username, agt_passwd, hostname, agt_port),
	    } 
	sp = ServerProxy(srv["agt_url"])
	
	try:
            rs = sp.sys_os_info()
            if not rs.startswith("success"): raise Exception(rs)
	    rs = parse_result(rs)
	    srv["arch"] = rs["arch"]
	    srv["cpu_num"] = int(rs["cpu_num"])
	    srv["cpu_speed"] = float(rs["bogomips"])
	    srv["mem_total"] = int(rs["mem_total"])
	    srv["status"] = "RUNNING"
	except Exception, e: #possible rpc error
	    errmsg = fmt_err_msg(e)
	    logger.warning("register_server: warning. hostname=%s %s" \
		    % (hostname, errmsg))
	    srv["status"] = "UNKOWN"

	db_update_srv(hostname, srv)
	logger.info("register_server: success. %s->%s" % (hostname, srv_role))
	return "success"
    except Exception, e: 
	errmsg = fmt_err_msg(e)
	logger.error("register_server: failed! %s->%s => %s" \
		% (hostname, srv_role, errmsg))
	return "failed:%s" % (errmsg,)

@exposed
def unregister_server(hostname):
    """
    unregister a server from site
    
    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	db_del_srv(hostname)
	logger.info("unregister_server: success. srv('%s') removed." % (hostname,))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("unregister_server: failed. srv('%s') => %s" % \
		(hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def register_vm(vm_path, vtech,     
	agt_username=DEFAULT_AGENT_USERNAME, 
	agt_passwd=DEFAULT_AGENT_PASSWD, 
	agt_port=DEFAULT_AGENT_PORT,
	packaged=False):
    """
    register vm

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	d = {
	    "vtech": vtech.lower().strip(),
	    "agt_addr": "",
	    "agt_username": agt_username,
	    "agt_passwd": agt_passwd,
	    "agt_port": agt_port,
	    "real_path": "",
	    "host_srv": "",
	    "packaged": packaged,
	    "status": "DOWN",	    
	    "readlock": 0,
	}

	db_update_vm(vm_path, d)
	logger.info("register_vm: success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("register_vm: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def unregister_vm(vm_path):
    """
    unregister vm

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	db_del_vm(vm_path)
	logger.info("unregister_vm: success. vm('%s')" % (vm_path,))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("unregister_vm: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_hostserver(vm_path):
    """
    get vm host server, if down, then hostname should be empty

    @return	    "success:host_srv=<hostname>" if success else "failed:<error message>" 
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)

	logger.info("get_vm_hostserver: success. vm('%s') => host_srv='%s'" % \
		(vm_path, vm['host_srv']))
	return "success:host_srv=%s" % vm["host_srv"]
    except Exception, e:
	errmsg = errmsg
	logger.error("get_vm_hostserver: error. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_path(host, vm_name, vtech="xen"):
    """
    get vm path by vm name from xenstore-*
    @return	    success:path=<val>

    N.B.
        !!deprecated!!
    """
    logger = logging.getLogger("ovs.query")
    try:
	vtech = vtech.lower().strip()
	srv = db_get_srv(host)
	sp = ServerProxy(srv['agt_url'])
	rs = getattr(sp, "%s_get_vm_path" % vtech)(vm_name)
	
	if rs.startswith('success'):
	    path = parse_result(rs)["path"]
	else:
	    raise Exception(rs)

	logger.info("get_vm_path: vm('%s') => %s" % (vm_name, path))
	return "success:path=%s" % path
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_path: failed! vm('%s') => %s" % (vm_name, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_ip_from_host(vm_path, hostname, vtech="xen"):
    """
    get vm ip by query
    N.B.
    !!deprecated!!
    """
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = getattr(sp, "%s_get_vm_mac" % vtech)(vm_path)
	ip = ""
        if not rs.startswith("success"): raise Exception(rs)
        d = parse_result(rs)
        for mac in d["mac"].split(","):
            ip = db_get_mac_ip(mac)
            if ip and host_alive(ip):
                vm = db_update_vm(vm_path, {"ip": ip})
                break

	logger.info("get_vm_ip_from_host:success. vm('%s') => ip=%s" % (vm_path, ip))
	return "success:ip=%s" % ip
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_ip_from_host:failed. vm('%s')=> %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

def get_live_vms(vtech="xen"):
    """
    query each host server for live vms, and update vm.db accordingly
    @return [vm1, vm2,] => list of running vms, 
    """
    logger = logging.getLogger("ovs.query")
    logger.info("get_live_vms:start...")
    
    vtech = vtech.lower().strip()
    collect_srv_stat()
    cds = CDS("srv")
    try:
        candidates = [hostname for (hostname, srv) in cds.db_srv.items() \
                if (vtech in srv["srv_role"] and srv["status"] == "RUNNING")]
    finally: cds.close()

    d_host_vms = {}
    for hostname in candidates:
        srv = db_get_srv(hostname)
        try:
            sp = ServerProxy(srv['agt_url'])
            rs = getattr(sp, "%s_get_vms_name_vncport_macs" % vtech)()
        except: continue
        if rs.startswith('success'):
            name_vncport_macs = parse_result(rs)["vm_info"]
            if not name_vncport_macs: 
                continue
            d_host_vms[hostname] = name_vncport_macs.split(",")
    
    #update db
    site_running_vms = []
    d_vm_in_doubt = {}
    cds = CDS("vm", "vmp", "macip")
    try:
        for hostname in d_host_vms:
            vm_infos = d_host_vms[hostname]
            for vm_info in vm_infos:
                vm_name, vncport, macs = vm_info.split("@") 
                site_running_vms.append(vm_name)

                #updated vm info => ip, vncport, host_srv
                if vncport: vncport = int(vncport)
                else: vncport = -1
                d = { "host_srv": hostname, "vnc_port": vncport }
                ip = ""
                for mac in macs.split("^"):
                    if mac in cds.db_macip:
                        ip = cds.db_macip[mac]
                        if ip: break
                d["ip"] = ip

                if vm_name in cds.db_vmp:
                    vm_path = cds.db_vmp[vm_name]
                    if vm_path in cds.db_vm:
                        vm = cds.db_vm[vm_path]
                        vm.update(d)
                        cds.db_vm[vm_path] = vm
                    else: #no entry in vm.db
                        d_vm_in_doubt[vm_name] = d
                else: #no entry in vmp.db 
                    d_vm_in_doubt[vm_name] = d
    finally: cds.close()
    #import vm (manage vm not created by ovs manager)
    #FIXME: possible conflict:
    # if user startup a vm outside OVS_BASE but with same name to a vm ovs managed. 
    for vm_name in d_vm_in_doubt:
        vm_info = d_vm_in_doubt[vm_name]
        vm_path = join(OVS_BASE, "running_pool/%s" % vm_name)
        try:
            real_path = find_real_path(vm_path)
            register_vm(vm_path, vtech)
            db_update_vm(vm_path, vm_info)
        except: #cannot get real_path
	    if vm_name in site_running_vms:
		site_running_vms.remove(vm_name)

    logger.info("get_live_vms: rs=%s" % repr(site_running_vms))
    return site_running_vms

def sync_vm_status(site_running_vms, agent_restart=True):
    """
    sync vm status based on info gethered from host servers

    @return site_running_vms
    """
    logger = logging.getLogger("ovs.query")
    logger.info("sync_vm_status: start...")

    #rebuild index vmp
    if agent_restart:
        cds = CDS("vm", "vmp")
        try:
            for vm_path in cds.db_vm:
                vm_name = basename(vm_path)
                cds.db_vmp[vm_name] = vm_path
        finally:
            cds.close()

    if not agent_restart:
        #routine sync, agent works normal
        #if not in running list, if status is RUNNING then set to DOWN
        l_in_doubt = []
        cds = CDS("vm", "vmp")
        try:
            vm_name_non_exist = []
            for vm_name in cds.db_vmp:
                vm_path = cds.db_vmp[vm_name]
                try:
                    vm = cds.db_vm[vm_path]
                    if vm["status"] == "RUNNING" and vm_name not in site_running_vms:
                        l_in_doubt.append(vm_path)
                except KeyError:
                    vm_name_non_exist.append(vm_name)
            for vm_name in vm_name_non_exist:
                del cds.db_vmp[vm_name]
        finally: cds.close()

        #double check if the vm in doubt is really running or down
        for vm_path in l_in_doubt:
            try:
                vm = db_get_vm(vm_path)
                srv = db_get_srv(vm['host_srv'])
                sp = ServerProxy(srv['agt_url'])
                rs = getattr(sp, "%s_get_vm_names" % vm['vtech'])() 
                vm_names = parse_result(rs)["vm_names"].split(",")
                vm_name = basename(vm_path)
                if vm_name not in vm_names: #not in running list indeed
		    logger.debug("sync_vm_status: vm('%s') RUNNING -> DOWN!" % vm_path)
		    db_update_vm(vm_path, {"status" : "DOWN"})
		else:
		    site_running_vms.append(vm_name)
            except Exception, e:
                logger.warning("sync_vm_status: vm('%s') => %s" % (vm_path, fmt_err_msg(e)))
	#sync DOWN -> RUNNING if in site_running_vms
	cds = CDS("vm", "vmp")
	try:
	    for vm_name in site_running_vms:
		try:
		    vm_path = cds.db_vmp[vm_name]
		    vm = cds.db_vm[vm_path]
		    if vm['status'] == "DOWN":
			vm['status'] = "RUNNING"
			cds.db_vm[vm_path] = vm
		except: 
		    pass
	finally: 
	    cds.close()

    else: 
        #called by agtctl startup
        #if in running list, set to RUNNING else DOWN
        cds = CDS("vm")
        try:
            for vm_path in cds.db_vm:
		try:
		    vm = cds.db_vm[vm_path]
		    vm_name = basename(vm_path)
		    if vm_name in site_running_vms:
			vm["status"] = "RUNNING"
		    else: 
			vm["status"] = "DOWN"
			vm["readlock"] = 0
		    cds.db_vm[vm_path] = vm
		except: pass
        finally: 
	    cds.close()

    logger.info("sync_vm_status: rs=%s" % repr(site_running_vms))
    return site_running_vms
    
@exposed   
def get_live_vm_info(agent_restart=False):
    """
    get info of all live vms, update the database

    @return	    "success:<string of host-info>" if success else "failed:<error message>"
    example of string of host-info: "host1=vm1:ip1:vnc_port1;host2=vm2:ip2:vnc_port2,vm3:ip3:vnc_port3"	
    string of host-info can be an empty string; ip/vnc_port can be an empty string.
    """
    logger = logging.getLogger("ovs.query")
    #make it ready to be called by utlis/do_rpc.py
    agent_restart = bool(agent_restart)
    try:
        site_running_vms = get_live_vms()
        site_running_vms = sync_vm_status(site_running_vms, agent_restart)
	
        host_vm_info = {}
        cds = CDS("vm", "vmp")
        try:
            for vm_name in site_running_vms:
                try:
                    vm_path = cds.db_vmp[vm_name]
                    vm = cds.db_vm[vm_path]
                    hostname = vm['host_srv']
                    if hostname:
                        if vm["vnc_port"] < 0: vnc_port = ""
                        else: vnc_port = "%d" % vm["vnc_port"]
                        if hostname in host_vm_info:
                            host_vm_info[hostname].append("%s:%s:%s" % (vm_name, vm["ip"], vnc_port))
                        else:
                            host_vm_info[hostname] = [ "%s:%s:%s" % (vm_name, vm["ip"], vnc_port), ]
                except Exception, e:
                    logging.debug("get_live_vm_info: warning. vm('%s') => %s" % \
                            (vm_name, fmt_err_msg(e)))
        finally: cds.close()
        rs = ""
        if host_vm_info:
            rs = ";".join([ "%s=%s" % (hostname, ",".join(vm_info)) 
                    for hostname, vm_info in host_vm_info.items() ])
	logger.info("get_live_vm_info: success. vm info => %s" % rs)
	return "success:%s" % rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_live_vm_info: failed. => %s" % errmsg)
	return "failed:%s" % errmsg

#----------------------------------------<> network config <>----
@exposed
def ifconfig_vm(vm_path, dev_name, inet_type, 
	ip=None, netmask=None, gateway=None):
    """
    config vm network interface

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	d = { dev_name: { "inet_type": inet_type, 
			  "ip": ip, 
			  "netmask": netmask, 
			  "gateway": gateway, } }
	
	vm = db_get_vm(vm_path)
	if "nics" not in vm: vm["nics"] = d
	else: vm["nics"].update(d)
	db_update_vm(vm_path, vm)	    

	logger.info("ifconfig_vm: success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("ifconfig_vm: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def hostname_vm(vm_path, hostname):
    """
    config vm hostname

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	db_update_vm(vm_path, {"hostname": hostname})
	logging.info("hostname_vm: success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("hostname_vm: success. vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_ip(vm_path):
    """
    get vm primary ip from site db

    @return	    "success:ip=<ip>" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	logger.info("get_vm_ip: vm('%s')->%s" % (vm_path, vm['ip']))
	return "success:ip=%s" % vm["ip"]
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_vm_ip: error! vm('%s')=>%s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def set_vm_status(vm_path, status):
    """
    @param status    valid status in ('DOWN', 'RUNNING', 'ERROR', ...) 
    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	db_update_vm(vm_path, {"status":status})
	logger.info("set_vm_status:success. vm('%s')=>%s" % (vm_path, status))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("set_vm_status:vm('%s') failed. => %s" % \
		(vm_path, errmsg(e)))
	return "failed:%s" % errmsg(e)

@exposed
def get_srv_agent_status(hostname):
    """
    @return	    "success:echo=<msg>" if success else "failed:<error message>" 
    """
    rs = "failed"
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.echo('hello')
	logger.info("get_srv_agent_status: agent('%s')=> %s" % (hostname, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	try: db_update_srv(hostname, {"status" : "UNKNOWN"})
	except: pass
	logger.error("get_srv_agent_status: error. agent('%s')=>%s" % \
		(hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vm_agent_status(vm_path):
    """
    @return	    "success:echo=<msg>" if success else "failed:<error message>" 
    N.B.
    !!deprecated!! we donnot install agent inside vm in this release
    """
    logger = logging.getLogger("ovs.query")
    try:
	vm = db_get_vm(vm_path)
	sp = ServerProxy(vm)
	rs = sp.echo('hello')
	logger.info("get_vm_agent_status: agent('%s')=> %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_agent_status: error. agent('%s')=>%s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

