#!/usr/bin/env python
#^#
# OVSSiteMonitor.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import logging
from OVSServerProxy import ServerProxy
from OVSDB import db_get_vm, db_update_vm
from OVSDB import db_get_srv, db_update_srv
from OVSCommons import exposed, fmt_err_msg, parse_result
from OVSCDS import CDS

@exposed
def get_hs_perf(hostname):
    """
    get host performance

    @return	    "success:<string of host-info>" if success else "failed:<error message>"
    example of string of host-info: "cpu_idle=<cpu idle>;mem_free=<free mem>;hd_avail=<harddisk avail>"
    """
    logger = logging.getLogger("ovs.performance")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.sys_perf_info()
	logger.info("get_hs_perf:srv('%s') => %s" % (hostname, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_hs_perf: error. srv('%s') => %s" % \
		(hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_servers_perf(hostnames=''):
    """
    within this site, get servers performance data, 
    hostnames is a list of hostname/ip delimited by ','
    
    @return	    "success:<string of srvs-info>" if success else "failed:<error message>"
    example of string of srvs-info: "srv1=cpu_idle1:mem_free1:hd_avail1,srv2=cpu_idle2:mem_free2:hd_avail2"
    """
    logger = logging.getLogger("ovs.performance")
    try:
        if hostnames.strip():
            hostnames = hostnames.split(",")
        else:
            cds = CDS("srv")
            try:
                hostnames = cds.db_srv.keys()
            finally:
                cds.close()

        results = []
        for hostname in hostnames:
            try:
                srv = db_get_srv(hostname)
                sp = ServerProxy(srv['agt_url'])
                rs = sp.sys_perf_info()
                if rs.startswith("success"):
                    d = parse_result(rs)
                    results.append("%s=%s:%s:%s" % \
                            (hostname, d["cpu_idle"], d["mem_free"], d["hd_avail"]))
                    db_update_srv(hostname, {"status" : "RUNNING"})
                else:
                    logger.error("get_server_perfs: srv('%s') => %s" % (hostname, rs))
                    results.append("%s=" % (hostname, ))
                    db_update_srv(hostname, {"status" : "UNKNOWN"})
            except Exception, e:
                logger.error("get_servers_perf: srv('%s') => %s" % \
                        (hostname, fmt_err_msg(e)))
                db_update_srv(hostname, {"status" : "UNKNOWN"})

        rs = "success:%s" % ( ";".join(results), )
        logger.info("get_servers_perf: rs=%s" % rs)
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("get_servers_perf: hostnames=%s => %s" % \
                (hostnames, errmsg))
        return "failed:%s" % errmsg

@exposed
def get_vm_perf(vm_path):
    """
    get vm performance, plz make sure the agent in the vm is running

    @return	    "success:<string of vm-info>" if success else "failed:<error message>"
    example of string of vm-info: "cpu_idle=<cpu idle>;mem_free=<free mem>;hd_avail=<harddisk avail>"

    N.B.
    !!deprecated!! agent is not installed inside vm in this release
    """
    logger = logging.getLogger("ovs.performance")
    try:
	vm = db_get_vm(vm_path)
	sp = ServerProxy(vm)
	rs = sp.sys_perf_info()
	logger.info("get_vm_perf:vm('%s') => %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_vm_perf: error. vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vms_perf(vtech='xen'):
    """
    within this site, get vms performance data, 

    @return	    "success:<vm-info>" if success else "failed:<error message>"
                    e.g. vm-info "vm1=cpu_idle1:mem_free1:hd_avail1;vm2=cpu_idle2:mem_free2:hd_avail2"
    """
    logger = logging.getLogger("ovs.performance")
    try:
	cds = CDS("srv")
	hostnames = [ hostname for hostname, srv in cds.db_srv.items() 
		if vtech in srv["srv_role"]]
	cds.close()

	results = []
	for hostname in hostnames:
	    try:
		srv = db_get_srv(hostname)
		sp = ServerProxy(srv['agt_url'])
		rs = getattr(sp, "%s_top_info" % vtech)()
		if rs.startswith("success"):
		    vm_perfs = parse_result(rs)
		    if vm_perfs: 
			vm_perfs = ["%s=%s" % (vm_name, perf) 
				for vm_name, perf in vm_perfs.items()]
			results.extend(vm_perfs)
		else:
		    logger.error("get_vms_perf: srv('%s') => %s" % (hostname, rs))
	    except Exception, e:
		logger.warning("get_vms_perf: srv('%s') => %s" % \
			(hostname, fmt_err_msg(e)))		    

	rs = "success:%s" %  ";".join(results) 
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_vms_perf: => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def get_host_info(hostname):
    """
    get architecture, cpu number, cpu speed (bogomips), memory size (in MB),

    @return     <refer to sys_os_info()>
    """
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.sys_os_info()
	logger.info("get_host_info: srv('%s') => %s" % (hostname, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_host_info: error. srv('%s') => %s" % \
		(hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_host_arch(hostname):
    """
    get cpu architecture

    @return	refer to sys_cpu_info()
    N.B.: to check if the server is suitable to act as the vm migrating target. 
    """
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.sys_cpu_info()

	logger.info("get_host_arch: srv('%s') => %s" % (hostname, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_host_arch: error. srv('%s') => %s" % (hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_hw_caps(hostname):
    """
    get hardware caps

    @return     <refer to xen_get_caps()>
    N.B.: use it to check if the server support hvm
    """
    logger = logging.getLogger("ovs.query")
    try:
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.xen_get_caps()

	logger.info("get_hw_caps: srv('%s') => %s" % (hostname, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logging.error("get_hw_caps: failed. srv('%s') => %s" % (hostname, errmsg))
	return "failed:%s" % errmsg
	

