#!/usr/bin/env python
#^#
# OVSSiteUtility.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
import re
from os.path import basename, dirname, join, exists, abspath
from os import makedirs
from OVSXUtility import find_real_path, utl_find_real_path, assemble_real_path, utl_get_free_space, get_storage_repos
from OVSXUtility import utl_list_dir, parse_storage_repos
from OVSServerSelection import select_one_server, get_utility_server
from OVSServerProxy import ServerProxy
from OVSDB import db_get_vm, db_get_srv, db_update_vm, db_del_vm, db_chkset_vm_status, db_get_real_vm_path
from OVSConstants import OVS_BASE
from OVSCommons import parse_result, exposed, chk_vm_path, fmt_err_msg, run_cmd

#------------------------------<> vm <>----
@exposed
def get_target_path(path):
    """
    get the real path after balance inside storage repositories 

    @return     "success:path=<val>" if success else "failed:<error message>"
    N.B. when cross-site cp vm, use this fucntion to get the target vm location
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(path)
        try:
            real_path = find_real_path(path, True)[0]
        except:
            (hostname, sp) = get_utility_server()
            rs = sp.utl_get_storage_repos()
            if not rs.startswith("success"): raise Exception(rs)
            rs = parse_result(rs)
            repos = rs["repos"]
            real_path = assemble_real_path(path, repos)
        
	logger.info("get_target_path: success. path('%s') => real_path('%s')" % (path, real_path))
        return "success:path=%s" % real_path
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_target_path: failed! path('%s') => %s" % (path, errmsg))
	return "failed:%s" % errmsg

@exposed
def _exists(vm_path):
    """
    check if a vm exists
    @return     "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)
	real_vm_path = find_real_path(vm_path)
	logger.info("_exists: success. vm_path('%s') => real_vm_path('%s')" % (vm_path, real_vm_path))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("_exists: failed! vm_path('%s') => %s" % (iso_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def is_bootable(iso_path):
    """
    check if a iso file is bootable

    @return	"success:bootable=<val>" if success else "failed:<error message>"
		val should be "yes" or "no"
    """
    logger = logging.getLogger("ovs.query")
    try:
	real_iso_path = find_real_path(iso_path)
	stdout = run_cmd("file %s" % real_iso_path)
	if stdout.strip().endswith("(bootable)"): bootable = "yes"
	else: bootable = "no"

	logger.info("is_bootable: success. iso('%s') => bootable('%s')" % (iso_path, bootable)) 
	return "success:bootable=%s" % bootable
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("is_bootable: failed! iso('%s') => %s" % (iso_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_vdisk_size(vdisk_path):
    """ 
    get vdisk size

    @return	    "success:size=<size>" if success else "failed:<error message>"
    block size is 1M
    """
    logger = logging.getLogger("ovs.query")
    try:
        from OVSXUtility import utl_get_vm_size
	rs = utl_get_vm_size(vdisk_path)

	logger.info("get_vdisk_size:vdisk('%s') => %s" % (vdisk_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vdisk_size: failed! vdisk('%s') => %s" % (vdisk_path, errmsg))
        return "failed:%s" % errmsg

@exposed
def get_vm_size(vm_path):
    """ 
    get vm size

    @return	    "success:size=<size>" if success else "failed:<error message>"
    block size is 1M
    """
    logger = logging.getLogger("ovs.query")
    try:
        real_vm_path = db_get_real_vm_path(vm_path)
        from OVSXUtility import utl_get_vm_size
	rs = utl_get_vm_size(real_vm_path)

	logger.info("get_vm_size:vm('%s') => %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("get_vm_size: failed! vm('%s') => %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def cp_vm(src_vm_path, dst_vm_path, 
	remote_server=None, rs_username=None, rs_passwd=None):
    """ 
    make a vm copy. this is the primary way to create a new vm. 
    if remote_server then cross-site copy

    @return	    "success:size=<vm size>" if success else "failed:<error message>" 
    """
    logger = logging.getLogger("ovs.operation")
    try:
	logger.info("cp_vm: '%s' -> '%s'" % (src_vm_path, dst_vm_path))
	chk_vm_path(src_vm_path)

	db_chkset_vm_status(src_vm_path, ("DOWN", "READLOCK"), "READLOCK")
	try:
	    (hostname, sp) = get_utility_server()
	    if not remote_server:
		chk_vm_path(dst_vm_path)
                src_real_vm_path = db_get_real_vm_path(src_vm_path)
                #srv vm size
                rs = sp.utl_get_vm_size(src_real_vm_path)
                if not rs.startswith("success"): raise Exception(rs)
                rs = parse_result(rs)
                vm_size = int(rs["size"])
                #target storage repository
                rs = sp.utl_get_storage_repos()
                if not rs.startswith("success"): raise Exception(rs)
                rs = parse_result(rs)
                repos, free_space = rs["repos"], int(rs["free_space"])
                #FIXME: still may run into problem when concurrent cps occur
                if free_space < vm_size + 10: 
                    raise Exception("no enough space under %s, vm_size=%d > free_space=%d" % \
                            (repos, vm_size, free_space))

                dst_real_vm_path = assemble_real_path(dst_vm_path, repos)
		rs = sp.utl_cp_vm(src_real_vm_path, dst_real_vm_path)
		logger.info("cp_vm: '%s' -> '%s' => %s" % (src_vm_path, dst_vm_path, rs))

		if rs.startswith("success"):
		    #update db
		    vm = db_get_vm(src_vm_path)
                    #duplicate vm info
		    db_update_vm(dst_vm_path, vm)
		    db_update_vm(dst_vm_path, {"status":"DOWN", "readlock": 0, "real_path": dst_real_vm_path})
	    else: 
		#N.B.: 
		# the caller is responsible to register this new vm to the remote site
                src_real_vm_path = db_get_real_vm_path(src_vm_path)
		rs = sp.utl_scp_vm(src_real_vm_path, dst_vm_path,remote_server, rs_username, rs_passwd)
		logger.info("cp_vm: '%s' -> '%s' => %s" % \
		    (src_vm_path, "%s@%s:%s" % (rs_username,remote_server, dst_vm_path), rs))
	finally:
            #for some reason, when db out sync, this may be more tolerance
	    #db_chkset_vm_status(src_vm_path, "READLOCK", "DOWN")
	    db_chkset_vm_status(src_vm_path, ("DOWN", "READLOCK"), "DOWN")

	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("cp_vm: failed! vm('%s') => %s" % (src_vm_path, errmsg))
	return "failed:%s" % (errmsg,)

@exposed
def rm_vm(vm_path):
    """
    remove vm

    @return	    "success" if success else "failed:<error message>" 
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	db_chkset_vm_status(vm_path, ("ERROR", "DOWN"), "PROCESSING")
	try:
            try:
                real_vm_path = db_get_real_vm_path(vm_path)
            except Exception, e:
                #FIXME: what if just mount point lost? 
                errmsg = fmt_err_msg(e)
	        logger.warning("rm_vm: cannot get real path vm('%s') => %s" % \
                        (vm_path, errmsg))
                db_del_vm(vm_path)
                return "success"
                
	    (hostname, sp) = get_utility_server()
	    rs = sp.utl_rm_vm(real_vm_path)
	    if rs.startswith("success"):
		db_del_vm(vm_path)
	    else:
		db_chkset_vm_status(vm_path, "PROCESSING", "ERROR")
	except Exception, e:
	    db_chkset_vm_status(vm_path, "PROCESSING", "ERROR")
	    raise e

	logger.info("rm_vm: vm('%s') => %s" % (vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("rm_vm: failed! => %s" % (errmsg,))
	return "failed:%s" % (errmsg,)

@exposed
def package_vm(vm_path, pkg_path):
    """
    Package a vm to a tarball

    @return	    "success:size=<pkg size>"; else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	logger.info("package_vm: '%s' -> '%s'" % (vm_path, pkg_path))
	chk_vm_path(vm_path)
	chk_vm_path(pkg_path)
	
	db_chkset_vm_status(vm_path, ("READLOCK", "DOWN"), "READLOCK")
	
	try:
            real_vm_path = db_get_real_vm_path(vm_path)
	    (hostname, sp) = get_utility_server()
            rs = sp.utl_get_storage_repos()
            if not rs.startswith("success"): raise Exception(rs)
            rs = parse_result(rs)
            repos, free_space = rs["repos"], int(rs["free_space"])
            #FIXME: how to check if free space is enough?
            real_pkg_path = assemble_real_path(pkg_path, repos)

	    rs = sp.utl_package_vm(real_vm_path, real_pkg_path)

	    if not rs.startswith("success"): raise Exception(rs)
            #update db
            vm = db_get_vm(vm_path)
            db_update_vm(pkg_path, vm)
            db_update_vm(pkg_path, {"status": "DOWN", "readlock": 0, 
                    "packaged": True, "real_path": real_pkg_path})
	finally:
	    db_chkset_vm_status(vm_path, ("DOWN", "READLOCK"), "DOWN")

	logger.info("package_vm: '%s' -> '%s' => %s" % (vm_path, pkg_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("package_vm: failed! '%s' -> '%s' => %s" % \
		(vm_path, pkg_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def extract_vm(pkg_path, vm_path):
    """
    extract a packaged vm, if pkg_path and vm_path are the same,
    the packaged vm will be deleted after operation.

    @return	    "success:size=<vm size>"; else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	logger.info("extract_vm: '%s' -> '%s'" % (pkg_path, vm_path))
	chk_vm_path(pkg_path)
	chk_vm_path(vm_path)

	if pkg_path != vm_path: #avoid delete at the same time
	    db_chkset_vm_status(pkg_path, ("READLOCK","DOWN"), "READLOCK")
	else: #extract to itself
	    db_chkset_vm_status(pkg_path, "DOWN", "PROCESSING")
	
	try:
            real_pkg_path = db_get_real_vm_path(pkg_path)

	    (hostname, sp) = get_utility_server()
            rs = sp.utl_get_storage_repos()
            if not rs.startswith("success"): raise Exception(rs)
            rs = parse_result(rs)
            repos, free_space = rs["repos"], int(rs["free_space"])
            #FIXME: how to check if free space is enough? 
            real_vm_path = assemble_real_path(vm_path, repos)
	    rs = sp.utl_extract_vm(real_pkg_path, real_vm_path)

	    if not rs.startswith("success"): raise Exception(rs)
            #update db
            vm = db_get_vm(pkg_path)
            db_update_vm(vm_path, vm)
            db_update_vm(vm_path, {"status": "DOWN", "readlock": 0, 
                    "packaged": False, "real_path": real_vm_path})
        
            #update vif mac addr
            rs1 = getattr(sp, "%s_clear_vif_mac" % vm["vtech"])(real_vm_path)
            if not rs1.startswith("success"):
                raise Exception(rs1)
	finally:
	    db_chkset_vm_status(pkg_path, ("PROCESSING", "DOWN", "READLOCK"), "DOWN")

	logger.info("extract_vm: '%s' -> '%s' => %s" % (pkg_path, vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("extract_vm: failed! '%s' -> '%s' => %s" % \
		(pkg_path, vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def mv_vm(old_vm_path, new_vm_path):
    """
    Move a vm to its new name
    
    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(old_vm_path)
	chk_vm_path(new_vm_path)

	db_chkset_vm_status(old_vm_path, ("ERROR", "DOWN"), "PROCESSING")
	try:
            old_real_vm_path = db_get_real_vm_path(old_vm_path)
            #here we assume they are within the same pool, 
            new_real_vm_path = join(dirname(old_real_vm_path), basename(new_vm_path))
	    (hostname, sp) = get_utility_server()
	    rs = sp.utl_mv_vm(old_real_vm_path, new_real_vm_path)

	    if not rs.startswith("success"): raise Exception(rs)

            #update db
            vm = db_get_vm(old_vm_path)
            db_update_vm(new_vm_path, vm)
            db_update_vm(new_vm_path, {"status": "DOWN", "readlock": 0, 
                    "real_path": new_real_vm_path})
            db_del_vm(old_vm_path)

	except Exception, e:
	    db_chkset_vm_status(old_vm_path, "PROCESSING", "ERROR")
	    raise e

	logger.info("mv_vm: '%s' -> '%s' => %s" % (old_vm_path, new_vm_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("mv_vm: failed! '%s' -> '%s' => %s" % \
                (old_vm_path, new_vm_path, errmsg))
	return "failed:%s" % errmsg

#--------------------------------------<> vm_disk <>---- 
@exposed
def create_vm_disk(vtech, disk_path, size, sparse=False):
    """
    create xen raw disk (a file filled with zeros)
    size is number of MB

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(disk_path)
	vtech = vtech.lower().strip()
	size = int(size)
        
        if "shareddisk" not in disk_path.lower():
            vm_path = dirname(disk_path)
            real_vm_path = find_real_path(vm_path)
            real_disk_path = join(real_vm_path, basename(disk_path))
        else:
            repos, freespace = get_storage_repos()
            real_disk_path = assemble_real_path(disk_path, repos)
        
        parent_dir = dirname(real_disk_path)
	if not exists(parent_dir): makedirs(parent_dir)
	rs = utl_get_free_space(parent_dir)
	if not rs.startswith("success"): 
            raise Exception(rs)
	free_space = int(parse_result(rs)['free_space'])
	if free_space < size: 
            raise Exception("not enough free space")

	#pick up one vm host server to create this disk
	(hostname, sp) = select_one_server(vtech)
	rs = getattr(sp, "%s_create_disk" % vtech)(real_disk_path, size, sparse)
	logger.info("create_vm_disk: disk('%s') size=%d => rs=%s" % \
		(disk_path, size, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("create_vm_disk: failed. disk('%s') size=%d => %s" % \
		(disk_path, size, errmsg))
	return "failed:%s" % errmsg

@exposed
def delete_vm_disk(vtech, disk_path):
    """
    remove a vm disk physically

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(disk_path)
	
        #FIXME: should set vtech to default value 'xen', swap the params 
	vtech = vtech.lower().strip()
        try:
            real_disk_paths = find_real_path(disk_path, True)
        except Exception, e: #should be "not found"
            logger.warning("delete_vm_disk: disk('%s') => %s" % \
                    (disk_path, fmt_err_msg(e)))
            return "success"

	#utility server will do the job
	(hostname, sp) = get_utility_server()
        for real_disk_path in real_disk_paths:
	    rs = getattr(sp, "%s_delete_disk" % vtech)(real_disk_path)
            if not rs.startswith("success"):
                raise Exception(rs)

	logger.info("delete_vm_disk: success. disk('%s')" % disk_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("delete_vm_disk: failed. disk('%s') => %s" % \
		(disk_path, errmsg))
	return "failed:%s" % errmsg

#--------------------------------------<> file system <>----
@exposed
def check_shared_storage(vm_path, hostname):
    """
    check if vm_path exists in the host

    @return	"success:result=<val>" if success else "failed:<error message>"
		val should be "yes" or "no"
    """
    logger = logging.getLogger("ovs.query")
    try:
	real_vm_path = db_get_real_vm_path(vm_path)
	srv = db_get_srv(hostname)
	sp = ServerProxy(srv['agt_url'])
	rs = sp.utl_find_real_path(vm_path)
	if not rs.startswith("success"):
	    rs = "no"
	else:
	    real_path = parse_result(rs)["real_path"]
	    if real_vm_path == real_path:
		rs = "yes"
	    else: rs = "no"

	logger.info("check_shared_storage: success. vm_path('%s'), hostname('%s') => %s" % \
		(vm_path, hostname, rs))
	return "success:result=%s" % rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("check_shared_storage: failed. vm_path('%s'), hostname('%s') => %s" % \
		(vm_path, hostname, errmsg))
	return "failed:%s" % errmsg

@exposed
def get_free_space():
    """
    get free space

    @return	    "success:free_space=<free space>" if success else "failed:<error message>"
    block size is 1M
    N.B.
        this will return maximum free space reside on one partition.
    """
    logger = logging.getLogger("ovs.query")
    try:
        storage, free_space = get_storage_repos()
        rs = "success:free_space=%d" % free_space

	logger.info("get_free_space: => %s" % rs)
	return rs     
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_free_space: failed. => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def get_free_space_vm(vm_path):
    """
    get free space

    @return	    "success:free_space=<free space>" if success else "failed:<error message>"
    block size is 1M
    N.B.
       vm of vm_path must already exists, for e.g. add a disk to a vm.
    """
    logger = logging.getLogger("ovs.query")
    try:
        real_vm_path = db_get_real_vm_path(vm_path)
        rs = utl_get_free_space(real_vm_path)

	logger.info("get_free_space: vm('%s') => %s" % (vm_path, rs))
	return rs     
    except Exception,e:
	errmsg = fmt_err_msg(e)
	logger.error("get_free_space: failed. vm('%s') => %s" % \
		(vm_path, errmsg))
	
@exposed
def list_dir(dir_name):
    """
    list all files and sub dirs in the dir

    @return	    "success:names=<string of names>" if success else "failed:<error message>"
    example of string of names: "file1,file2,file3,dir1/,dir2/"
    a name ends with '/' is a directory, else it is a file.
    """
    logger = logging.getLogger("ovs.query")
    try:
	d_name = abspath(dir_name)
        ptn = re.compile("^%s" % OVS_BASE)
        possible_paths = [ptn.sub(join(OVS_BASE, "local"), d_name)]
	possible_paths.append(d_name)
        d = parse_storage_repos()
        possible_paths.extend([ptn.sub(join(OVS_BASE, uuid), d_name) for uuid in d])
        real_paths = [path for path in possible_paths if exists(path)]
	
	items = []
	for real_path in real_paths:
	    rs = utl_list_dir(real_path)
	    if not rs.startswith("success"):
		raise Exception(rs)
	    else:
		names = parse_result(rs)["names"]
		items.append(names)
	rs = ",".join(items)

	logger.info("list_dir: directory('%s') => %s" % (dir_name, rs))
	return "success:names=%s" % rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("list_dir: failed. directory('%s') => %s" % \
		(dir_name, errmsg))
	return "failed:%s" % errmsg

@exposed
def make_dir(path):
    """
    make dir

    @return	"success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	(hostname, sp) = get_utility_server()
	rs = sp.utl_make_dir(path)

	logger.info("make_dir: directory('%s') => %s" % (path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("make_dir: failed. directory('%s') => %s" % \
		(path, errmsg))
	return "failed:%s" % errmsg

@exposed
def remove(path, remove_all=True):
    """
    remove a file/tree

    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	(hostname, sp) = get_utility_server()
        if remove_all:
	    real_paths = find_real_path(path, True)
            for real_path in real_paths:
                rs = sp.utl_remove(real_path)
                if not rs.startswith("success"): break
        else:
	    real_path = find_real_path(path, False)
            rs = sp.utl_remove(real_path)

        if not rs.startswith("success"):
            raise Exception(rs)  
	logger.info("remove: success. => '%s'" % path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("remove: failed. %s => %s" % (path, errmsg))
	return "failed:%s" % errmsg

def move(old_path, new_path):
    """
    move a file/dir to its new path

    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	(hostname, sp) = get_utility_server()
	rs = sp.utl_move(old_path, new_path)

	logger.info("move: %s to %s => %s" % (old_path, new_path, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("move: failed. %s to %s => %s" % (old_path, new_path, errmsg))
	return "failed:%s" % errmsg

#--------------------------------------<> wget <>---- 
@exposed
def test_url(url, proxy_server="", proxy_user="", proxy_password=""):
    """
    test spcified url
    
    @para proxy_server e.g. http://www-proxy.us.oracle.com
    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	(hostname, sp) = get_utility_server()
	rs = sp.utl_test_url(url, proxy_server, proxy_user, proxy_password)
	if not rs.startswith("success"):
	    raise Exception(rs)

	logger.info("test_url: success. url('%s') => %s" % (url, rs))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("test_url: failed. url('%s') => %s" % (url, errmsg))
	return "failed:%s" % errmsg

@exposed
def wget(url, path, is_dir=True, name="", proxy_server="", proxy_user="", proxy_password=""):
    """
    @para url	    e.g. http://host/path
    @para path	    path to save the dir/file
    @para is_dir    True if it is a dir, False if it is a file.
    @para name	    new name for the dir/file, if it is a empty string, do not rename the dir/file.
    @para proxy_server e.g. http://www-proxy.us.oracle.com

    @return	    "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
        (hostname, sp) = get_utility_server()
        logger.info("server('%s') is selected as utility server" % hostname)
	
	found = False
	if not is_dir: #download a file
            try:
                real_path = find_real_path(path, True)[0]
                found = True
	    except: #if not exists will create it later
                pass
	if not found:	
	    rs = sp.utl_get_storage_repos()
	    if not rs.startswith("success"): raise Exception(rs)
	    rs = parse_result(rs)
	    repos = rs["repos"]
	    real_path = assemble_real_path(path, repos)

	rs = sp.utl_wget(url, real_path, is_dir, name, proxy_server, proxy_user, proxy_password)
	if not rs.startswith("success"):
	    raise Exception(rs)

	logger.info("wget: success. url('%s') => path('%s'), name('%s')" % (url, path, name))
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("wget: failed. url('%s'), path('%s'), name('%s') => %s" % \
		(url, path, name, errmsg))
	return "failed:%s" % errmsg
