#!/usr/bin/env python

#^#
# OVSXUtility.py
#
# ovs agent module for cp/rm/mv/scp etc
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import urllib
import sys
import string
import re
import logging
from shutil import copytree, copyfile, rmtree, move
from os.path import join, isdir, isfile, islink, exists, dirname, basename, abspath, isabs
from os import makedirs, stat, walk, remove, rename, system 
from glob import glob
from OVSConstants import STORAGE_REPO_CONFIG, OVS_BASE	
from OVSCommons import parse_result, run_cmd, fmt_err_msg, chk_vm_path, exposed
from OVSXXenVMConfig import xen_clear_vif_mac

#----------------------------<> helper function <>----
def get_storage_free_space(dir_path):
    logger = logging.getLogger("ovs.query")

    d = {}
    try: 
        if not exists(dir_path): 
            raise Exception("dir('%s') does not exist!" % dir_path)
        stdout = run_cmd("df --block-size=M %s" % dir_path)
    except Exception, e: 
        logger.warning("get_storage_free_space: failed. '%s' =>%s" % \
                (dir_path, fmt_err_msg(e)))
        return d

    lines = stdout.splitlines()
    #Mounted on, mp = mount point
    mp_idx0 = lines[0].index("Mounted on")
    #Available
    avail_idx0 = lines[0].index("Used") + 5
    avail_idx1 = lines[0].index("Use%")
    
    l = []
    success_mapping = False
    for idx, line in enumerate(lines[1:]):
        try:
            if line.startswith(" "): #previous line is for Filesystem
                if success_mapping == True: #and filesystem line done a success mapping
                    del l[-1]
            success_mapping = False

            mp = line[mp_idx0:].strip()
            hd_avail = line[avail_idx0 : avail_idx1].strip(' M')
            l.append((mp, int(hd_avail)))

            success_mapping = True
        except: continue
        
    return dict(l)

def parse_storage_repos():
    #ovs-makerepo created repos
    d = {}
    try:
        f = file(STORAGE_REPO_CONFIG)
        for line in f:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            try:
                uuid, fs = string.split(line, maxsplit=1)
                d[uuid] = fs
            except: continue
        f.close()
    except:#possible file does not exist 
        pass

    return d

def find_real_path(vpath, list_all=False):
    """
    find the real location of path, which could be inside any shared dir,
    created by ovs-makerepo.
    for e.g.:
    '/OVS/running_pool/a' could be one and only one of these paths:
    /OVS/local/running_pool/a
    /OVS/<uuid1>/running_pool/a
    /OVS/<uuid2>/running_pool/a
    
    @list_all   if True will return list if multiple paths found 
                else will raise exception. 
    ...
    """
    logger = logging.getLogger("ovs.query")
    
    vpath = abspath(vpath)
    possible_paths = [vpath]

    ptn = re.compile("^%s" % OVS_BASE)
    possible_paths.append(ptn.sub(join(OVS_BASE, "local"), vpath))

    d = parse_storage_repos()
    possible_paths.extend([ptn.sub(join(OVS_BASE, uuid), vpath) for uuid in d])
    real_paths = [path for path in possible_paths if exists(path)]
    real_paths = list(set(real_paths))
    if real_paths: 
        if not list_all:
            if len(real_paths) > 1: 
                raise Exception("Got more than one file/dir with the same name: '%s'" % repr(real_paths))
            return real_paths[0]
        else:
            return real_paths
    else:
        raise Exception("Could not find '%s' in any storage repository." % vpath)

@exposed
def utl_find_real_path(vpath, list_all=False):
    """
    find the real location of path, which could be inside any shared dir,
    created by ovs-makerepo.
    for e.g.:
    '/OVS/running_pool/a' could be one and only one of these paths:
    /OVS/local/running_pool/a
    /OVS/<uuid1>/running_pool/a
    /OVS/<uuid2>/running_pool/a
    """
    logger = logging.getLogger("ovs.query")
    try:
        list_all = bool(list_all)
	rs = find_real_path(vpath, list_all)
        if list_all: rs = ",".join(rs)    
	logger.info("utl_find_real_path: success. path('%s') => %s" % (vpath, rs))
	return "success:real_path=%s" % rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("utl_find_real_path: failed. =>%s" % errmsg) 
        return "failed:%s" % errmsg

def get_storage_repos():
    #default location
    #path = join(OVS_BASE, "local")
    path = OVS_BASE
    #if not exists(path): makedirs(path)
    d = get_storage_free_space(path)
    #may not be a mount point 
    d_repos_space = { path : d.values()[0] }

    #SharedSR 
    d = parse_storage_repos()
    sr_dirs = [join(OVS_BASE, uuid) for uuid in d]
    for sr_dir in sr_dirs:
        d = get_storage_free_space(sr_dir)
        if sr_dir in d: #sr_dir must be a mount point 
            d_repos_space[sr_dir] = d[sr_dir]
    
    #no SharedSR
    if len(d_repos_space) == 1: 
        return d_repos_space.items()[0]
    #result is repos with maximum space
    d = dict([(space, repos) for repos, space in d_repos_space.items()])
    free_space = max(d_repos_space.values())
    repos = d[free_space]
    return repos, free_space

def assemble_real_path(vpath, repos):
    return re.sub(r"^%s" % OVS_BASE, repos, vpath)

#--------------------<> VM manipulation <>---- 
@exposed
def utl_get_storage_repos():
    logger = logging.getLogger("ovs.query")
    try:
	repos, free_space = get_storage_repos()

        rs = "success:repos=%s;free_space=%d" % (repos, free_space)
        logger.info("util_get_storage_repos: %s" % rs)
        return rs 
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("utl_get_storage_repos: failed. =>%s" % errmsg) 
        return "failed:%s" % errmsg

@exposed
def utl_cp_vm(src_vm_path, dst_vm_path):
    """
    copy vm 
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(src_vm_path)
	chk_vm_path(dst_vm_path)

	logger.info("utl_cp_vm: vm('%s') -> vm('%s') start..." % (src_vm_path, dst_vm_path))

	if not exists(src_vm_path): 
	    raise Exception("VM('%s') does not exist." % src_vm_path)
	if exists(dst_vm_path): 
	    raise Exception("VM('%s') already exists." % dst_vm_path)

	parent_dir = dirname(dst_vm_path.rstrip("\\/"))
	if not exists(parent_dir):
		makedirs(parent_dir)
	if isfile(src_vm_path):
	    copyfile(src_vm_path, dst_vm_path)
	elif isdir(src_vm_path):
	    copytree(src_vm_path, dst_vm_path)
            #clear mac for new vm
            dst_vm_cfg_file = join(dst_vm_path, "vm.cfg")
            if exists(dst_vm_cfg_file):
                rs = xen_clear_vif_mac(dst_vm_path)
                if not rs.startswith("success"): 
                    raise Exception(rs)
                                
	rs = utl_get_vm_size(dst_vm_path)
	logger.info("utl_cp_vm: vm (%s) created!" % dst_vm_path)
	return rs
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_cp_vm: failed! => %s" % errmsg)
	return "failed:%s" % errmsg

@exposed
def utl_rm_vm(vm_path):
    """
    remove vm
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)

	if not exists(vm_path): 
	    logger.warning("utl_rm_vm: vm (%s) not exist!" % vm_path)
	    return "success:warning=>vm (%s) not exist!" % vm_path 
	
	if isdir(vm_path):
	    #allow to remove the vm when creating failed
	    #and (exists(join(vm_path, "vm.xml")) or exists(join(vm_path, "vm.cfg"))):
	    rmtree(vm_path)
	elif isfile(vm_path): # and vm_path.endswith("tar.gz"): 
	    remove(vm_path)
	else:
	    raise Exception("Not a vm. (config file does not exist.)")
	logger.info("utl_rm_vm: vm (%s) deleleted!" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_rm_vm: failed! vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_mv_vm(old_vm_path, new_vm_path):
    """
    Move a vm to its new name
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(old_vm_path)
	chk_vm_path(new_vm_path)

	if not exists(old_vm_path): raise Exception("VM('%s') does not exist." % old_vm_path)
	if exists(new_vm_path): raise Exception("VM('%s') already exists." % new_vm_path)

	parent_dir = dirname(new_vm_path.rstrip("\\/"))
	if not exists(parent_dir):
	    makedirs(parent_dir)

	move(old_vm_path, new_vm_path)
	logger.info("utl_mv_vm: success. vm('%s') -> vm('%s')" % (old_vm_path, new_vm_path))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_mv_vm: failed! vm('%s') -> vm('%s') => %s" % \
		(old_vm_path, new_vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_package_vm(vm_path, pkg_path):
    """
    Package vm to a tarball
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(vm_path)
	chk_vm_path(pkg_path)

	parent_dir = dirname(pkg_path.rstrip("\\/"))
	if not exists(parent_dir):
	    makedirs(parent_dir)

	logger.info("utl_package_vm: vm('%s') -> '%s' => start..." % (vm_path, pkg_path))
	if not pkg_path.endswith("tar.gz"): 
	    pkg_path = "%s.tar.gz" % pkg_path.rstrip("\\/")
	vm_path = abspath(vm_path)
	cmd = "cd \"%s\" && tar -zcf \"%s\" ./" % (vm_path, pkg_path)
	run_cmd(cmd)

	logger.info("utl_package_vm: success. cmd='%s'" % cmd)
	return utl_get_vm_size(pkg_path)
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_package_vm:failed! vm('%s') -> '%s' => %s" % \
		(vm_path, pkg_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_extract_vm(pkg_path, vm_path):
    """
    extract a packaged vm
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(pkg_path)
	chk_vm_path(vm_path)

	logger.info("utl_extract_vm: '%s' -> '%s' => start..." % (pkg_path, vm_path))
	#if not pkg_path.endswith("tar.gz"): pkg_path = "%s.tar.gz" % pkg_path
	if not isfile(pkg_path): raise Exception("Packaged VM('%s') does not exist." % pkg_path)
	if isdir(vm_path): raise Exception("vm('%s') already exists!" % vm_path)

	delete_after = False
	if pkg_path == vm_path:
	    delete_after = True
	    move(pkg_path, "%s.tar.gz" % pkg_path) 
	    pkg_path = "%s.tar.gz" % pkg_path

	makedirs(vm_path)
	cmd = "tar -zxf %s -C %s" % (pkg_path, vm_path)
	run_cmd(cmd)

	logger.info("utl_extract_vm: success. cmd='%s'" % cmd)
	if delete_after: remove(pkg_path)
	return utl_get_vm_size(vm_path)
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_extract_vm:failed! '%s' -> vm('%s') => %s" % \
		(pkg_path, vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_scp_vm(src_vm_path, dst_vm_path, remote_server, rs_username, rs_passwd):
    """
    cross-site vm copy
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(src_vm_path)
	
	#TODO: how to check dst_vm_path?
	dst_vm_path = "%s@%s:%s" % (rs_username, remote_server, dst_vm_path)
	
	#the trailing / in a source name means "copy the contents of this directory". 
	#Without the trailing / "copy the directory". 
	if isfile(src_vm_path):
	    cmd = "%s %s %s" % (join(dirname(__file__),"utils/rcp.py"), src_vm_path, dst_vm_path)
	elif isdir(src_vm_path):
	    cmd = "%s %s/ %s" % (join(dirname(__file__),"utils/rcp.py"), src_vm_path, dst_vm_path)
	else:
	    #bug#3813, when src vm path is invalid, possible caused by 
	    #removing manually or mount point geting stale 
	    raise Exception("Src VM('%s') is not file or dir." % src_vm_path)
	logger.info("utl_scp_vm: cmd='%s'" % cmd) 
	run_cmd("%s %s" % (cmd, rs_passwd))

	logger.info("utl_scp_vm: success. cmd='%s'" % cmd)
	return utl_get_vm_size(src_vm_path)
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_scp_vm:failed! %s" % errmsg)
	return "failed:%s" % errmsg

#------------------------------<> exposed helper function <>----
@exposed
def utl_get_vm_size(vm_path):
    """
    get the size of the vm
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(vm_path)

	if exists(vm_path):
            cmd = "du --apparent-size --summarize --block-size=1M --one-file-system %s" % vm_path
	    stdout = run_cmd(cmd)
	    size = stdout.split("\t")[0]
	else: 
            raise Exception("vm('%s') does not exist!" % vm_path)

	logger.info("utl_get_vm_size: vm('%s') => %s" % (vm_path, size)) 
	return "success:size=%s" % size
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_get_vm_size: failed! vm('%s') => %s" % (vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_list_dir(dir_name):
    """
    list all files and sub dirs in the dir
    
    N.B.
       ',' or '=' is not allowed in file/dir name, or it will cause 
    misbehavior for the result parser 
    """
    names = []
    logger = logging.getLogger("ovs.query")
    try:
	if isdir(dir_name):
	    for root, dirs, files in walk(dir_name):
		for f in files:
		    if not f.startswith('.'): names.append(f)
		for d in dirs:
		    if not d.startswith('.'): names.append('%s/' % d)
		break
	    name_str = ','.join(names)
	else:
	    raise Exception("utl_list_dir: failed! '%s' is not a directory" % dir_name)
	
	logger.info("utl_list_dir: directory('%s') => %s" % (dir_name, name_str))
	return "success:names=%s" % name_str
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_list_dir: failed! directory('%s') => %s" % (dir_name, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_exists(path):
    """
    tell if a file is exists

    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.query")
    try:
	chk_vm_path(path)
	if exists(path): 
	    logger.info("utl_exists: success. path='%s'" % path)
	    return "success"
	else: raise Exception("Does not exist.")
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_exists:failed. path='%s' => %s" % (path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_get_free_space(path):
    """
    get free disk space
    """
    logger = logging.getLogger("ovs.query")
    try:
        if not exists(path):
            raise Exception("dir('%s') does not exist!" % path)
	stdout = run_cmd("df -B M %s" % path)
	lines = stdout.splitlines()
	idx0 = lines[0].index("Used") + 5
	idx1 = lines[0].index("Use%")
	hd_avail = lines[-1][idx0:idx1].strip(' M')

	logger.info("utl_get_free_space: success. path('%s') => %s" % (path, hd_avail))
	return "success:free_space=%s" % hd_avail
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_get_free_space: failed. path('%s') => %s" % (path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_make_dir(path):
    """
    make dir
    """
    logger = logging.getLogger("ovs.operation")
    try:
	path = abspath(path)
	chk_vm_path(path)
	if not path.startswith(OVS_BASE):
	    raise Exception("Please make sure the path is under '%s'." % OVS_BASE)
	if not isdir(path):
	    makedirs(path)

	logger.info("utl_make_dir: success. directory('%s') created" % path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_make_dir: failed. directory('%s') => %s" % (path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_remove(path):
    """
    remove a tree/file
    """
    logger = logging.getLogger("ovs.operation")
    try:
	path = abspath(path)
	chk_vm_path(path)
	if not path.startswith(OVS_BASE):
	    raise Exception("Please make sure the path is under '%s'." % OVS_BASE)

	if isdir(path):
	    rmtree(path)
	else:
	    remove(path)

	logger.info("utl_remove: success. %s removed" % path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_remove: failed. %s => %s" % (path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_move(old_path, new_path):
    """
    move a dir/file to its new path
    """
    logger = logging.getLogger("ovs.operation")
    try:
	old_path = abspath(old_path)
	new_path = abspath(new_path)
	chk_vm_path(old_path)
	chk_vm_path(new_path)

	if not old_path.startswith(OVS_BASE) or not new_path.startswith(OVS_BASE):
	    raise Exception("Please make sure the paths are under %s." % OVS_BASE)

	move(old_path, new_path)

	logger.info("utl_move: success. %s moved to %s" % (old_path, new_path))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_move: failed. %s to %s => %s" % \
		(old_path, new_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_create_disk(disk_path, size, sparse=True):
    """
    create xen raw disk (a file filled with zeros)
    size is number of MB
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(disk_path)

	if exists(disk_path):
	    return "failed:file already exists."
	pdir = dirname(disk_path)
	if not isdir(pdir): makedirs(pdir)
	
	if sparse: seek = size - 1; count = 1
	else: seek = 0; count = size 
	cmd = "ionice -c3 dd if=/dev/zero of=%s oflag=direct bs=1M seek=%d count=%d" \
		% (disk_path, seek, count)
	run_cmd(cmd)

	logger.info("xen_create_disk: success. disk('%s')" % disk_path)
	return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_create_disk: error. disk('%s')=>%s" % (disk_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_delete_disk(disk_path):
    """
    delete disk physically
    """
    logger = logging.getLogger("ovs.operation")
    try:
	chk_vm_path(disk_path)

        if isdir(disk_path):
            raise Exception("xen disk should be a file, but ('%s') is a dir." % disk_path)
	if not isfile(disk_path): 
	    logger.warning("xen_delete_disk: disk('%s') not exist." % disk_path)
	    return "success"
	else:
	    #we assume xen disk consists only one single file
	    remove(disk_path)
	    logger.info("xen_delete_disk: success. disk('%s')"  % disk_path)	    
	return "success"
    except Exception, e:
        errmsg = fmt_err_msg(e)
	logger.error("xen_delete_disk: failed. disk('%s') => %s" % (disk_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_test_url(url, proxy_server="", proxy_user="", proxy_password=""):
    """
    test url
    """
    logger = logging.getLogger("ovs.operation")
    try:
	if not proxy_server:
	    cmd = "wget --spider %s" % url
	else:
	    lower_url = url.lower()
	    if lower_url.startswith("http://"):
		proxy = "http_proxy"
	    elif lower_url.startswith("ftp://"):
		proxy = "ftp_proxy"
	    elif lower_url.startswith("https://"):
		proxy = "https_proxy"
	    else:
		raise Exception("Invalid url('%s')." % url)
	    
	    if not proxy_user:
		cmd = "export %s='%s' && wget --spider '%s'" % \
			(proxy, proxy_server, url)
	    else:
		cmd = "export %s='%s' && wget --proxy-user='%s' --proxy-password='%s' --spider '%s'" % \
			(proxy, proxy_server, proxy_user, proxy_password, url)

	run_cmd(cmd)
	logger.info("utl_test_url: success. url('%s')" % url)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_test_url: failed. url('%s') => %s" % (url, errmsg))
	return "failed:%s" % errmsg

@exposed
def utl_wget(url, path, is_dir=True, name="", proxy_server="", proxy_user="", proxy_password=""):
    """
    download dir/file with wget
    """
    logger = logging.getLogger("ovs.operation")
    try:
        chk_vm_path(path)
	path = abspath(path)
	if is_dir:
	    if name == "":
		name = url.strip('/').split('/')[-1]
	    if not url.endswith('/'):
		url = "%s/" % url
	    reject = "*.html*"
	    parameter = "--quiet --recursive --level=1 --continue --tries=3 \
		    --no-parent --no-directories --reject=%s \
		    --directory-prefix=%s/%s %s" % \
		    (reject, path, name, url)
	    if not proxy_server:
		cmd = "wget %s" % parameter
	    else:
		lower_url = url.lower()
		if lower_url.startswith("http://"):
		    proxy = "http_proxy"
		elif lower_url.startswith("ftp://"):
		    proxy = "ftp_proxy"
		elif lower_url.startswith("https://"):
		    proxy = "https_proxy"
		else:
		    raise Exception("Invalid url('%s')" % url)
		
		if not proxy_user:
		    cmd = "export %s=%s && ionice -c3 wget %s" % \
			    (proxy, proxy_server, parameter)
		else:
		    cmd = "export %s=%s && ionice -c3 wget --proxy-user=%s --proxy-password=% %s" % \
			    (proxy, proxy_server, proxy_user, proxy_password, parameter)

	    logger.debug("wget cmd: %s" % cmd)
	    rs = system(cmd)
	    if rs:
		raise Exception("error when processing wget")
	else:
	    if not proxy_server:
		cmd = "ionice -c3 wget -P %s %s" % (path, url)
	    else:
		lower_url = url.lower()
		if lower_url.startswith("http://"):
		    proxy = "http_proxy"
		elif lower_url.startswith("ftp://"):
		    proxy = "ftp_proxy"
		elif lower_url.startswith("https://"):
		    proxy = "https_proxy"
		else:
		    raise Exception("Invalid url('%s')" % url)

		if not proxy_user:
		    cmd = "export %s=%s && ionice -c3 wget -P %s %s" % \
			    (proxy, proxy_server, path, url)
		else:
		    cmd = "export %s=%s && ionice -c3 wget --proxy-user=%s --proxy-password=%s -P %s %s" % \
			    (proxy, proxy_server, proxy_user, proxy_password, path, url)

	    logger.debug("utl_wget: cmd=%s" % cmd)
	    rs = system(cmd)
	    if rs:
		raise Exception("error when processing wget")

	    #move
	    if not name == "":
	       file = urllib.unquote(url.strip('/').split('/')[-1])
	       oldpath = "%s/%s" % (path, file)
	       newpath = "%s/%s" % (path, name)
	       rs = utl_move(oldpath, newpath)
	       if not rs.startswith("success"):
		   raise Exception(rs)
	    
	logger.info("utl_wget: success. url('%s') => path('%s'), name('%s')" % \
		(url, path, name))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("utl_wget: failed. url('%s') to path('%s'), name('%s') => %s" % \
		(url, path, name, errmsg))
	return "failed:%s" % errmsg

