#!/usr/bin/env python
#^#
# OVSXXenVMInstall.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#
import string
import re
import os
import logging
from tempfile import NamedTemporaryFile
from shutil import copyfile
from os.path import join, isdir, isfile, basename, abspath
from OVSCommons import fmt_err_msg, run_cmd, exposed, parse_result
from OVSXXenVMConfig import xen_unmarshall_cfg, xen_marshall_cfg, xen_set_vm_vnc_password
from OVSXXenStore import xen_mem_free
from OVSXUtility import utl_exists

#----------------------------<> helper function <>----
def parse_vif_cfg(vif_cfg):
    macs, bridges = [], []
    for vif in vif_cfg.split(";"):
        try:
            mac, bridge = map(string.strip, vif.split(","))
            if mac: macs.append(mac)
            if bridge: bridges.append(bridge)
        except: pass
    if len(macs) == len(bridges):
        return zip(macs, bridges)
    else:
        return bridges

#----------------------------<> install from scratch <>----
@exposed
def xen_install_vm_hvm(vm_path, memory, vcpus, vif_cfg, disks, cdrom, vncpassword, vnclisten="0.0.0.0"):
    """
    install full virtualized vm from scratch
    @param memory	size in MB
    @param vcpus	cpu number
    @param vif_cfg      refer to xen_set_vifs()
    @param disks	disk string, in format "disk_path:disk_size:is_sparse" e.g.
			system.raw:2:False
		    disk path should be relative to vm_path
		    disk_size is counted in GB
		    is_sparse should be True or False
    @param cdrom	path to the install media iso file
    @return		"success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	vm_name = basename(vm_path)
	if not re.match(r"^[0-9a-zA-Z_]+$", vm_name): 
	    raise Exception("Domain name must be alphanumeric or _")
	if len(vm_name) > 64:
	    raise Exception("Length of Domain name must be less than 64")
	if not isdir(vm_path): 
	    os.makedirs(vm_path)

	rs = utl_exists(cdrom)
	if not rs.startswith("success"):
	    raise Exception("cdrom('%s') does not exist." % cdrom)

	rs = xen_mem_free()
	if not rs.startswith("success"):
	    raise Exception(rs)
	rs = parse_result(rs)
	mem_free = int(rs["mem_free"])
	if mem_free < memory:
	    raise Exception("not enough memory")

	disks = disks.split(",")
	disks = [disk.split(":") for disk in disks]
	disk_param = ""
	for disk_path, disk_size, sparse_option in disks:
	    disk_param += "--file %s --file-size=%s " % \
		    (join(vm_path, disk_path), disk_size)
	    if sparse_option.lower() == "false":
		disk_param += " --nonsparse "
        
        vif_param = ""
        rs = parse_vif_cfg(vif_cfg)
	for e in rs:
            if isinstance(e, str):
                bridge = e
	        vif_param += " --bridge=%s " % bridge
            else:
                mac, bridge = e
                vif_param += " --mac=%s --bridge=%s " % (mac, bridge)

	f = NamedTemporaryFile()
	try:
	    f.write(vncpassword)
	    f.flush()
	    cmd = """virt-install --hvm --usbtablet --name=%s --ram=%d --vcpus=%d %s %s \
		    --vnc --vnclisten=%s --vncpasswd=%s --noautoconsole --keep-cdrom --cdrom=%s""" % \
		    (vm_name, int(memory), int(vcpus), vif_param, disk_param, vnclisten, f.name, cdrom)
	    logger.debug("xen_install_vm_hvm: cmd=%s" % cmd)
	    run_cmd(cmd)
	finally: f.close()

	vm_cfg_file = join(vm_path, "vm.cfg")
	copyfile("/etc/xen/%s" % vm_name, vm_cfg_file)
	cfg = xen_unmarshall_cfg(vm_cfg_file)
	if "uuid" in cfg: del cfg["uuid"]
	xen_marshall_cfg(cfg, vm_cfg_file)
	xen_set_vm_vnc_password(vm_path, vncpassword)

	logger.info("xen_install_vm_hvm:success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_install_vm_hvm: error. vm('%s') %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_change_vm_cdrom(vm_path, iso_path):
    """
    change vm cdrom
    @param  iso_path abspath to iso file
    @return "success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	vm_name = basename(vm_path)
	be_dev = "file:%s" % iso_path
	fe_dev = "hdc:cdrom"
	
#	#try to detach
#	cmd = "xm block-configure %s phy:/dev/null %s r; sleep 2" % \
#		(vm_name, fe_dev)
#	try: run_cmd(cmd)
#	except Exception, e:
#	    logger.debug("xen_change_vm_cdrom:error.vm('%s')=>%s" % \
#		    (vm_path, fmt_err_msg(e)))

	#attach
	cmd = "xm block-configure %s %s %s r" % \
		(vm_name, be_dev, fe_dev)
	run_cmd(cmd)

	logger.info("xen_change_vm_cdrom:success. vm('%s') iso('%s')" % \
		(vm_path, iso_path))
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_change_vm_cdrom:failed. vm('%s') iso('%s')=>%s" % \
		(vm_path, iso_path, errmsg))
	return "failed:%s" % errmsg

@exposed
def xen_install_vm_para(vm_path, memory, vcpus, vif_cfg, disks, location, vncpassword, vnclisten="0.0.0.0"):
    """
    install para virtualized vm from scratch
    @param memory	size in MB
    @param vcpus	cpu number
    @param vif_cfg      refer to xen_set_vifs()
    @param disks	disk string, in format "disk_path:disk_size:is_sparse" e.g.
			system.raw:2:False
		    disk path should be relative to vm_path
		    disk_size is counted in GB
		    is_sparse should be True or False
    @param location	location of the install directory e.g.
			nfs:host:path
			http and ftp protocols are also supported e.g.
			http://host/path
			ftp://host/path
    @return		"success" if success else "failed:<error message>"
    """
    logger = logging.getLogger("ovs.operation")
    try:
	vm_name = basename(vm_path)
	if not re.match(r"^[0-9a-zA-Z_]+$", vm_name): 
	    raise Exception("Domain name must be alphanumeric or _")
	if len(vm_name) > 64:
	    raise Exception("Length of Domain name must be less than 64")
	if not isdir(vm_path):
	    os.makedirs(vm_path)

	rs = xen_mem_free()
	if not rs.startswith("success"):
	    raise Exception(rs)
	rs = parse_result(rs)
	mem_free = int(rs["mem_free"])
	if mem_free < memory:
	    raise Exception("not enough memory")
	
	disks = disks.split(",")
	disks = [disk.split(":") for disk in disks]
	disk_param = ""
	for disk_path, disk_size, sparse_option in disks:
	    disk_param += "--file %s --file-size=%s " % \
		    (join(vm_path, disk_path), disk_size)
	    if sparse_option.lower() == "false":
		disk_param += " --nonsparse "

        vif_param = ""
	rs = parse_vif_cfg(vif_cfg)
	for e in rs:
            if isinstance(e, str):
                bridge = e
	        vif_param += " --bridge=%s " % bridge
            else:
                mac, bridge = e
                vif_param += " --mac=%s --bridge=%s " % (mac, bridge)
	
	f = NamedTemporaryFile()
	try:
	    f.write(vncpassword)
	    f.flush()
	    cmd = """virt-install --paravirt --name=%s --ram=%d --vcpus=%d %s %s --vnc \
		    --vnclisten=%s --vncpasswd=%s --noautoconsole --location=%s""" % \
		    (vm_name, int(memory), int(vcpus), vif_param, disk_param, vnclisten, f.name, location)

	    logger.debug("xen_install_vm_para: cmd=%s" % cmd)
	    run_cmd(cmd)
	finally: f.close()

	vm_cfg_file = join(vm_path, "vm.cfg")
	copyfile("/etc/xen/%s" % vm_name, vm_cfg_file)
	cfg = xen_unmarshall_cfg(vm_cfg_file)
	if "uuid" in cfg: del cfg["uuid"]
	xen_marshall_cfg(cfg, vm_cfg_file)
	xen_set_vm_vnc_password(vm_path, vncpassword)

	logger.info("xen_install_vm_para:success. vm('%s')" % vm_path)
	return "success"
    except Exception, e:
	errmsg = fmt_err_msg(e)
	logger.error("xen_install_vm_para: error. vm('%s') %s" % \
		(vm_path, errmsg))
	return "failed:%s" % errmsg
        
